INTL_SWO_AO_COLLECTOR = "collector.appoptics.com"
INTL_SWO_AO_STG_COLLECTOR = "collector-stg.appoptics.com"
INTL_SWO_COMMA = ","
INTL_SWO_COMMA_W3C_SANITIZED = "...."
INTL_SWO_EQUALS = "="
INTL_SWO_EQUALS_W3C_SANITIZED = "####"
INTL_SWO_TRACESTATE_KEY = "sw"
INTL_SWO_X_OPTIONS_KEY = "sw_xtraceoptions"
INTL_SWO_SIGNATURE_KEY = "sw_signature"
INTL_SWO_DEFAULT_TRACES_EXPORTER = "solarwinds_exporter"
INTL_SWO_TRACECONTEXT_PROPAGATOR = "tracecontext"
INTL_SWO_PROPAGATOR = "solarwinds_propagator"
INTL_SWO_DEFAULT_PROPAGATORS = [
    INTL_SWO_TRACECONTEXT_PROPAGATOR,
    "baggage",
    INTL_SWO_PROPAGATOR,
]
INTL_SWO_DOC_SUPPORTED_PLATFORMS = 'https://documentation.solarwinds.com/en/success_center/observability/default.htm#cshid=app-sysreqs-python-agent'
INTL_SWO_DOC_TRACING_PYTHON = 'https://documentation.solarwinds.com/en/success_center/observability/default.htm#cshid=app-add-python-agent'
INTL_SWO_SUPPORT_EMAIL = 'SWO-support@solarwinds.com'